/* 
 *  TOPPERS ATK2
 *      Toyohashi Open Platform for Embedded Real-Time Systems
 *      Automotive Kernel Version 2
 *  
 *  Copyright (C) 2007-2008 by TAKAGI Nobuhisa
 *  Copyright (C) 2011-2012 by Center for Embedded Computing Systems
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2011-2012 by FUJISOFT INCORPORATED, JAPAN
 *  Copyright (C) 2011-2012 by FUJITSU VLSI LIMITED, JAPAN
 *  Copyright (C) 2011-2012 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2011-2012 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2011-2012 by Renesas Electronics Corporation, JAPAN
 *  Copyright (C) 2011-2012 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2011-2012 by TOSHIBA CORPOTATION, JAPAN
 *  Copyright (C) 2011-2012 by Witz Corporation, JAPAN
 *  
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 *  
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 * 
 *  $Id: checker.cpp 133 2012-12-18 04:30:30Z ertl-hiro $
 */
 
#include <fstream>
#include <map>
#include "toppers/workaround.hpp"
#include "toppers/s_record.hpp"
#include "toppers/nm_symbol.hpp"
#include "toppers/misc.hpp"
#include "toppers/diagnostics.hpp"
#include "toppers/xml/checker.hpp"
#include "toppers/xml/cfg1_out.hpp"

namespace toppers
{
  namespace xml
  {

    struct checker::implementation
    {
    protected:
      std::tr1::shared_ptr< s_record > srec_;
      std::tr1::shared_ptr< nm_symbol > syms_;

      virtual ~implementation() {}
      virtual bool do_check( cfg1_out& cfg1out, std::map< std::string, std::size_t > const& count_map )
      {
        return true;
      }
      virtual implementation* do_clone() const
      {
        return new implementation( *this );
      }

      friend class checker;
    };

    checker::checker()
      : pimpl_( new implementation )
    {
    }

    /*!
     *  \brief  ԡ󥹥ȥ饯
     *  \param[in]  ԡ
     */
    checker::checker( checker const& other )
      : pimpl_( other.pimpl_->do_clone() )
    {
    }

    //! ǥȥ饯
    checker::~checker()
    {
      delete pimpl_;
      pimpl_ = 0;
    }

    /*!
     *  \brief  ROM᡼Υ
     *  \param[in]  srec_file   S쥳ɷΥե̾
     *  \param[in]  nm_file     nmޥɤϤܥơ֥Υե̾
     */
    void checker::load_rom_image( std::string const& srec_file, std::string const& nm_file )
    {
      std::ifstream ifs_srec( srec_file.c_str() );
      if ( !ifs_srec.is_open() )
      {
        fatal( _( "cannot open file `%1%\'" ), srec_file );
      }
      std::tr1::shared_ptr< s_record > srec( new s_record( ifs_srec ) );

      std::ifstream ifs_nm( nm_file.c_str() );
      if ( !ifs_nm.is_open() )
      {
        fatal( _( "cannot open file `%1%\'" ), nm_file );
      }
      std::tr1::shared_ptr< nm_symbol > syms( new nm_symbol( ifs_nm ) );

      pimpl_->srec_.swap( srec );
      pimpl_->syms_.swap( syms );
    }

    /*!
     *  \brief  åμ»
     *  \param[in]  cfg1_out
     */
    bool checker::check( cfg1_out& cfg1out ) const
    {
      //cfg1_out::static_api_map api_map( cfg1out.merge() );

      std::map< std::string, std::size_t > count_map;
/*
for ( cfg1_out::static_api_map::const_iterator iter( api_map.begin() ), last( api_map.end() );
            iter != last;
            ++iter )
      {
        std::size_t count = 0;
        for ( std::vector< static_api >::const_iterator iter2( iter->second.begin() ), last2( iter->second.end() );
              iter2 != last2;
              ++iter2 )
        {
          if ( !iter2->get_info()->slave )
          {
            ++count;
          }
        }
        count_map[ iter->first ] = count;
      }
*/
      return pimpl_->do_check( cfg1out, count_map );
    }

    /*!
     *  \brief  ܥθ
     *  \param[in]  symbol  륷ܥ̾
     *  \return     
     *
     *  ˼Ԥˤϡֵͤ.typeեɤ-1ꤵ롣
     */
    nm_symbol::entry checker::find( std::string const& symbol ) const
    {
      return pimpl_->syms_->find( symbol );
    }

    /*!
     *  \brief  ꥢɥ쥹ͤ
     *  \param[in]  address   ɥ쥹
     *  \param[in]  size      ͤХȿ
     *  \param[in]  little_endian   Хȥȥ륨ǥʤtrueꤹ
     *  \return     ̵֤ͤ
     */
    std::tr1::uintmax_t checker::get( std::size_t address, std::size_t size, bool little_endian ) const
    {
      return pimpl_->srec_->get_value( address, size, little_endian );
    }

  }
}
